
#include <omega/pres_subs.h>

Substitutions::Substitutions(Relation &input_R, Conjunct *input_c) {
    int i;
    r = new Relation(input_R,input_c);
    c = r->single_conjunct();
    c->reorder_for_print();
    c->ordered_elimination(r->global_decls()->length());
    int num_subs = c->problem->nSUBs; 
    subs = new eqn[num_subs];
    for(i = 0; i < num_subs; i++)
	subs[i] = SUBs[i];
    subbed_vars.reallocate(num_subs);
    /* Go through and categorize variables as:
         1) substituted, 2) not substituted, 3) wildcard
       Safevars number of variables were not able to be substituted.
       nVars number of total variables, including wildcards.
       nSUBs is the number of substitutions.
       nSUBs + nVars == the number of variables that went in. */
    Variable_ID_Tuple unsubbed_vars(c->problem->safeVars);
    for(i = 1; i <= c->mappedVars.size(); i++) 
	if(c->mappedVars[i]->kind() != Wildcard_Var) {
	    int addr = c->problem->forwardingAddress[i];
	    assert(addr == c->find_column(c->mappedVars[i]) && addr != 0);
	    if(addr < 0) {
		assert(-addr <= subbed_vars.size());
		subbed_vars[-addr] = c->mappedVars[i];
	    } else {
		assert(addr <= unsubbed_vars.size());
		unsubbed_vars[addr] = c->mappedVars[i];
	    }
	} else {
	    // Here we don't redeclare wildcards, just re-use them.
	    unsubbed_vars.append(c->mappedVars[i]);
	}
    assert(unsubbed_vars.size() + subbed_vars.size() == c->mappedVars.size());
    c->mappedVars = unsubbed_vars; /* These are the variables that remain */
}

Substitutions::~Substitutions() {
    delete [] subs;
    delete r;
}

bool Substitutions::substituted(Variable_ID v) {
    return (subbed_vars.index(v) > 0);
}

Sub_Handle Substitutions::get_sub(Variable_ID v){
    assert(substituted(v) && "No substitution for variable");
    return Sub_Handle(this,subbed_vars.index(v)-1,v);
}


//::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::


int Sub_Iterator::live() const {
  return current <= last;
}

void Sub_Iterator::operator++() { this->operator++(0); }

void Sub_Iterator::operator++(int) {
    current++;
}

Sub_Handle Sub_Iterator::operator*() {
  assert(s && current <= last && "Sub_Iterator::operator*: bad call");
  return Sub_Handle(s,current,s->subbed_vars[current+1]);
} 

Sub_Handle Sub_Iterator::operator*() const {
  assert(s && current <= last && "Sub_Iterator::operator*: bad call");
  return Sub_Handle(s,current,s->subbed_vars[current+1]);
} 


//::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::



Sub_Handle::Sub_Handle(Substitutions *_s, int _e, Variable_ID _v) :
Constraint_Handle(_s->c,&(_s->subs),_e), v(_v) {}

String Sub_Handle::print_to_string() const {
    return c->problem->print_EQ_to_string(&((*eqns)[e]));
}

String Sub_Handle::print_term_to_string() const {
    return c->print_term_to_string(&(*eqns[e]));
}
